[−][src]Trait embedded_time::clock::Clock
The Clock trait provides an abstraction of hardware-specific timer peripherals, external timer
devices, RTCs, etc.
The Clock is characterized by an inner unsigned integer storage type (either u32 or
u64), a u32/u32 fraction (Period) defining the duration (in seconds) of one
count of this Clock, and a custom error type (must implement crate::Error) representing
errors that may be generated by the implementation.
In addition to the Clock::now() method which returns an Instant, an unlimited number of
software Timers can be spawned from a single Clock instance.
Associated Types
type Rep: TimeInt
The type to hold the tick count
type ImplError: Error
Implementation-specific error type
This type can be returned using the Error::Other(E)
Associated Constants
Loading content...Required methods
fn now(&self) -> Result<Instant<Self>, Error<Self::ImplError>>
Get the current Instant
Errors
Implementation-specific error returned as
Error::Other(Self::ImplError)
Provided methods
fn new_timer<Dur: Duration>(
&self,
duration: Dur
) -> Timer<OneShot, Armed, Self, Dur>
&self,
duration: Dur
) -> Timer<OneShot, Armed, Self, Dur>
Spawn a new, OneShot Timer from this clock