[−][src]Module embedded_time::duration
Duration types/units creation and conversion.
Traits
| Duration | An unsigned duration of time |
| TryConvertFrom | Attempt to convert from one duration type to another |
| TryConvertInto | Attempt to convert from one duration type to another |